require "mod-gui"
require "util"
require "utils"
require "item_prices"
require "tech_prices"
require "about"
local format_time = util.formattime
local format_number = util.format_number
local rand = math.random


-- ******************************************************************************************************
-- ******************************************************************************************************
--   This is the *JailBreak* Scenario
--               by MFerrari
--
--  Suggestions: - play with Cats mod, You will have to protect and rescue cute cats.
--               - play with "RPG System" Mod. Each round survived here gives XP to the survivors.
--   also, to add more challenge, consider using the Explosive-Biters mod
--   if any other mods that adds enemies prototypes, they will also be used here.
-- ******************************************************************************************************
-- ******************************************************************************************************


-- Settings
STARTING_MONEY  = 1000


function FormatEvolution()
 return {"evolution_factor", string.format("%.2f", math.floor(game.forces["enemy"].evolution_factor * 10000) / 100)}
end


function init_globals()

	global.jb_seed      = global.jb_seed or math.floor(tonumber(tostring({}):sub(8,-4)))
	global.jb_generator = global.jb_generator or game.create_random_generator(global.jb_seed )

global.jb_Round        = 0
global.jb_BossRound    = 5 -- default 5
global.jb_TickToKill   = 0 
global.jb_TimeToExecute= 3*60*60
global.jb_TickToCleanWorms = 0
global.jb_TimeToNextRound= 0
global.jb_MarkedTick= 0
global.jb_RestTime     = 6*60*60
global.jb_RestTimeBoss = 11*60*60
global.jb_TeamMoney    = STARTING_MONEY
global.jb_UnitKills    = 0
global.jb_SpawnerKills = 0
global.jb_WormKills    = 0
global.jb_WhereYouRespawn = {}
global.HoldPlayerItems = {}
global.jb_ChestLootToDestroy = {}
global.skipped_multiplier = 0.1
global.Executioner_name = "big-worm-turret"
global.JB_MAX_ROUNDS       = 40
global.GP_DEATH_PENALTY    = 5     --%
global.EVOLUTION_PER_KILL  = 0.01  -- enemy evolve when executing prisoners
global.EVOLUTION_PER_ROUND = 0.04  
global.GP_Mult = 1
global.Wave_Power_Mult = 1.0
global.JB_GROUP_LIMIT  = 20
global.MIN_TO_CASH_OUT = 1000
global.MONEY_PER_ROUND = 500
global.XP_PER_ROUND = 300 -- for RPG System mod only
global.jb_Enemy_Power_Bonus  = 0
global.jb_NextEvents = {}

global.jb_CatNames =   -- mod Cats
   {
   'small-cat',
   'medium-cat',
   }

CreatePricesList()
CreateTechPricesList()
read_about_map()
end

function insert_items(entity, array)
  for name, count in pairs (array) do
    entity.insert({name = name, count = count})
  end
end

function scenario_starting_equipment(player)
  local items = 
  {
    ["iron-gear-wheel"] = 500,
    ["transport-belt"] = 600,
    ["underground-belt"] = 40,
    ["splitter"] = 40,
    ["gun-turret"] = 10,
    ["stone-wall"] = 50,
    ["repair-pack"] = 40,
    ["inserter"] = 100,
--    ["burner-inserter"] = 50,
    ["small-electric-pole"] = 50,
    ["medium-electric-pole"] = 50,
    ["big-electric-pole"] = 25,
--    ["burner-mining-drill"] = 50,
    ["electric-mining-drill"] = 50,
    ["stone-furnace"] = 35,
    ["steel-furnace"] = 20,
--    ["electric-furnace"] = 8,
    ["assembling-machine-1"] = 50,
    ["assembling-machine-2"] = 20,
--    ["assembling-machine-3"] = 8,
    ["electronic-circuit"] = 200,
    ["fast-inserter"] = 100,
    ["long-handed-inserter"] = 100,
    ["substation"] = 10,
    ["boiler"] = 10,
    ["offshore-pump"] = 1,
    ["steam-engine"] = 20,
    ["chemical-plant"] = 20,
    ["oil-refinery"] = 5,
    ["pumpjack"] = 10,
    ["small-lamp"] = 20,
    ["science-pack-1"] = 200,
	["science-pack-2"] = 200,
	["small-lamp"] = 20,
--	["power-armor"] = 2,
--	["fusion-reactor-equipment"]=2,
--	["personal-roboport-equipment"]=2,
    ["roboport"]=3,
	["logistic-chest-storage"]=2,
	["construction-robot"] = 50,
  }
  insert_items(player, items)
end


function give_starting_equipment(player)
  local items = 
  {
    ["iron-plate"] = 200,
    ["pipe"] = 200,
    ["pipe-to-ground"] = 50,
    ["copper-plate"] = 200,
    ["steel-plate"] = 200,
    ["iron-gear-wheel"] = 100,
    ["transport-belt"] = 50,
    ["underground-belt"] = 20,
    ["splitter"] = 10,
    ["gun-turret"] = 2,
    ["stone-wall"] = 20,
    ["repair-pack"] = 20,
    ["inserter"] = 20,
    ["small-electric-pole"] = 30,
    ["steel-chest"] = 10,	
  }
  insert_items(player, items)
end


function give_spawn_equipment(player)
  local items = 
    {
      ["steel-axe"] = 3,
      ["submachine-gun"] = 1,
      ["firearm-magazine"] = 40,
      ["shotgun"] = 1,
      ["shotgun-shell"] = 20,
	  ["heavy-armor"] = 1,
 --     ["power-armor"] = 1,
--     ["construction-robot"] = 20,
--      ["blueprint"] = 3,
--      ["deconstruction-planner"] = 1
    }
  insert_items(player, items)
--[[  local equipment = 
    {
      "fusion-reactor-equipment",
      "exoskeleton-equipment",
      "personal-roboport-equipment",
      "personal-roboport-equipment"
    }
  local armor = player.get_inventory(5)[1].grid
  for k, name in pairs (equipment) do
    armor.put({name = name})
  end
  for k, equipment in pairs (armor.equipment) do
    equipment.energy = equipment.max_energy
  end]]
end


function setup_techs()

local tech_ini = {
'automation',
'logistics',
'steel-processing',
'electronics',
'automation-2',
'optics',
'stone-walls',
'turrets',
'military',
'heavy-armor',
'fluid-handling',
'oil-processing',
'electric-energy-accumulators-1',
'electric-energy-distribution-1',
'logistics-2',
'advanced-material-processing',
'circuit-network',
'land-mine',
'landfill',
'gates',
}

for t, tech in pairs(tech_ini) do 
	game.forces.player.technologies[tech].researched=true 
	end

local tech_disable = {
'rail-signals',
'railway',
'artillery',
'artillery-shell-range-1',
'artillery-shell-speed-1',
'fluid-wagon',
'automated-rail-transportation',

}

for t, tech in pairs(tech_disable) do 
	game.forces.player.technologies[tech].enabled=false
	end

end



-- Give XP to mod RPG System
function GiveTeamXP(XP)
 if remote.interfaces["RPG"] then
	remote.call("RPG","TeamXP","player",XP)
   end
end




function init_unit_settings()
  game.map_settings.pollution.enabled = false
  game.map_settings.unit_group.max_group_slowdown_factor = 1
  game.map_settings.unit_group.max_member_speedup_when_behind = 2
  game.map_settings.unit_group.max_member_slowdown_when_ahead = 0.8
  game.map_settings.unit_group.member_disown_distance = 25
  game.map_settings.unit_group.min_group_radius = 8.0
  game.map_settings.path_finder.use_path_cache = true
  game.map_settings.path_finder.max_steps_worked_per_tick = 300
  game.map_settings.path_finder.short_cache_size = 50
  game.map_settings.path_finder.long_cache_size = 50
  
 --https://lua-api.factorio.com/latest/LuaEntityPrototype.html
 --result_units :: array of UnitSpawnDefinition [R]	The result units and spawn points with weight and evolution factor for a biter spawner entity.
--           {unit="small-biter", spawn_points={{evolution_factor=0.0, weight=0.3}, {0.7, 0.0}}}
  
  
  
  local Units={}
  local Spawners={}
  local Worms={}
  for K,proto  in pairs (game.entity_prototypes) do
	if proto.result_units then -- found a spawner
		table.insert(Spawners, proto.name) 
		for Y,RU in pairs (proto.result_units) do
			table.insert(Units, RU) 
			end
		end
	if proto.turret_range and proto.autoplace_specification  then
		table.insert(Worms, proto.name)
		end
   end
  global.jb_Spawners = Spawners
  global.jb_Units    = Units
  global.jb_Worms	 = Worms
end



function HideExecutioners()
  for k, entity in pairs (game.surfaces[1].find_entities_filtered{
                          name = global.Executioner_name, 
						  area={global.jb_Executioners[1],global.jb_Executioners[#global.jb_Executioners]}}) do
      entity.destroy()
  end
global.jb_TickToCleanWorms=0

RestorePlayerItems(game.players[1])
end



function RaiseExecutioners()
for k=1, #global.jb_Executioners do
	local position = global.jb_Executioners[k]
	game.surfaces[1].create_entity{name="explosion", position=position, force=game.forces.neutral}
	local worm = game.surfaces[1].create_entity{name = global.Executioner_name, position = position, force=game.forces.enemy}
	worm.destructible = false
	end
global.jb_jail_timer.destroy()	
global.jb_jail_timer=nil
global.jb_TickToKill=0	
global.jb_TickToCleanWorms=game.tick + 30*60
--game.print('Execution begins!',colors.red)
end


function check_Replicator()
local spent =0
local inv = global.jb_Replicator.get_inventory(defines.inventory.chest)
local invdest = global.jb_Destinator.get_inventory(defines.inventory.chest)
    for i = 1, #inv do
        local stack = inv[i]
		if stack and stack.valid and stack.valid_for_read then
			local copy_of_item_stack = { name = stack.name, count = stack.count*2, health = stack.health or nil, durability = stack.durability or nil }
			local count = copy_of_item_stack.count/2
			local price = global.Market_Item_Prices[stack.name]
			if price == nil then 
				global.jb_Replicator.surface.spill_item_stack(global.jb_Replicator.position, stack, true)
				inv.remove(stack)
				break
				end
			price = price * count 	
			if price> global.jb_TeamMoney then 
			   game.surfaces[1].create_entity{name = "flying-text", position = global.jb_Replicator.position, text = "$$$$ ????", color = colors.red}
				global.jb_Replicator.surface.spill_item_stack(global.jb_Replicator.position, stack, true)
				inv.remove(stack)
			   break
			   end
			if invdest.can_insert(copy_of_item_stack) then
				local inserted = invdest.insert(copy_of_item_stack)
				local amt_not_inserted = stack.count - inserted
--				if amt_not_inserted > 0 then
					--player.surface.spill_item_stack(player.position, items, true)
	--				end
				global.jb_TeamMoney = global.jb_TeamMoney - price
				spent = spent + price 
				else
				  game.surfaces[1].create_entity{name = "flying-text", position = global.jb_Destinator.position, text = "<==  full?", color = colors.red}
				  global.jb_Replicator.surface.spill_item_stack(global.jb_Replicator.position, stack, true) 
				  inv.remove(stack)
				  break
				end
			inv.remove(stack)
		end
	end

if spent >0 then
	game.surfaces[1].create_entity{name = "flying-text", position = global.jb_Replicator.position, text = "- $ " ..spent, color = colors.yellow}
	game.surfaces[1].create_entity{name = "flying-text", position = global.jb_Destinator.position, text = "+ + + +", color = colors.yellow}	
	end
end




function check_SellerChest()
local spent =0
local inv = global.jb_SellerChest.get_inventory(defines.inventory.chest)
    for i = 1, #inv do
        local stack = inv[i]
		if stack and stack.valid and stack.valid_for_read then
			local copy_of_item_stack = { name = stack.name, count = stack.count, health = stack.health or nil, durability = stack.durability or nil }
			local count = copy_of_item_stack.count
			local price = global.Market_Item_Prices[stack.name]
			if price == nil then 
				global.jb_SellerChest.surface.spill_item_stack(global.jb_SellerChest.position, stack, true)
				else
				price = math.floor(price/3 * count)
				spent = spent + price
				global.jb_TeamMoney = global.jb_TeamMoney + price
				end
			inv.remove(stack)
		end
	end

if spent >0 then
	game.surfaces[1].create_entity{name = "flying-text", position = global.jb_SellerChest.position, text = "+ $ " ..spent, color = colors.yellow}
	end
end


function update_jail_timer ()
if global.jb_jail_timer and global.jb_jail_timer.valid then
   global.jb_jail_timer.text = time_to_that_tick(global.jb_TickToKill)
   else
   local timer = game.surfaces[1].create_entity{name = "flying-text",
         time_to_live=global.jb_TimeToExecute, 
		 position = global.jb_Jail_position,
		 text = time_to_that_tick(global.jb_TickToKill),
		 color = colors.red,		 
		 speed = 0.05}
   timer.active = false	 
   global.jb_jail_timer = timer	 
   end
end   


function on_tick(event)


if global.jb_TickToKill>0 then
   if game.tick % 60*60 == 0 then update_jail_timer() end
   if game.tick>global.jb_TickToKill then
		RaiseExecutioners() end  end 
		
if global.jb_TickToCleanWorms>0 then
   if game.tick>global.jb_TickToCleanWorms then
		HideExecutioners() end  end 

if global.jb_TickToKill>0 or global.jb_TickToCleanWorms>0 then
   if event.tick % 1000 == 0 then check_jail_prisoners() end
   end
	
if global.jb_TimeToNextRound>0 then
   if game.tick>global.jb_TimeToNextRound then
      RoundStart() return  end
   else  -- ROUND RUNNING
   if event.tick % 350 == 0 then
      check_aliens_alive() return end
   if event.tick % 7500 == 0 then
      command_straglers() end
   if is_boss_round() then
        if game.tick - global.jb_MarkedTick > global.jb_RestTimeBoss * 2 then finish_round_now() return  
		elseif event.tick % (60*25) == 0 then CallFrenzyAttack() end
	  else 
	    if game.tick - global.jb_MarkedTick > global.jb_RestTime * 2 then finish_round_now() return  end
	  end	   
   end
	
  if event.tick % 1800 == 0 then
    check_Replicator() 
	check_SellerChest()
    check_cats_alive()
	end
	
  if event.tick % 60 == 0 then
    update_connected_players()
	if #global.jb_NextEvents > 0 then check_next_events() end
	end
		
end
script.on_event(defines.events.on_tick, on_tick )



function check_next_events()

for e,eve in pairs (global.jb_NextEvents) do
	local event = eve.event 
	local tick  = eve.tick
	local tab   = eve.tab_event

	if game.tick > tick then
	
		if event=='add_mapflag' then
		    local txt = tab.text
			local ico = tab.icon  --{type="item",name="rocket-silo"}
			local pos = tab.position
			local tag = game.forces["player"].add_chart_tag(game.surfaces[1],{icon=ico, text=txt,position=pos}) --{"missions.silo-loc"}
			end
		table.remove(global.jb_NextEvents,e)
		end
	end
end


function setup_executioners()
  local surface = game.surfaces[1]
  local turrets = {}
  local spawns = {}
  local waypoints = {}
  
  for k, entity in pairs (surface.find_entities_filtered{name = "big-worm-turret"}) do
    local position = entity.position
    table.insert(turrets, position)
    entity.destroy()
  end

  for k, entity in pairs (surface.find_entities_filtered{name = "spitter-spawner"}) do
    local position = entity.position
      table.insert(waypoints, position)
    entity.destroy()
  end
  
  for k, entity in pairs (surface.find_entities_filtered{name = "biter-spawner"}) do
    local position = entity.position
      table.insert(spawns, position)
    entity.destroy()
  end

  global.jb_Executioners = turrets
  global.jb_Spawns_Pos   = spawns
  global.jb_Waypoints    = waypoints
  
end


function set_market_offers()

local market = global.jb_Market
local surface = market.surface									
local offers = global.Market_Items
local roundoffers = {}

for k=25,1,-1 do 
    market.remove_market_item(k) end

local r = math.ceil(math.max(global.jb_Round,2) * #offers / global.JB_MAX_ROUNDS)
local ma = math.min(r+30,#offers)
local mi = math.max(ma-30,1)

if ma-mi<30 then 
   ma = #offers-30
   mi = ma - 50
   end

for x=mi,ma do
    table.insert(roundoffers,offers[x])
	end

if #roundoffers<20 then return end   
local market_items_to_add = math.random(15,25)
	while market_items_to_add >= 0 do		
			--local i = jb_Rand(mi,ma)   --  travando algumas vezes 
        local i = math.random(1,#roundoffers)
		if roundoffers[i] then
		    local offer = roundoffers[i]
			local price = offer.price[1][2]
			price = math.max(math.ceil(price * rand(5,9)/10  ),1)
			
			offer.price[1][2]=price
			market.add_market_item(offer)
			market_items_to_add = market_items_to_add - 1
			table.remove(roundoffers,i)
			end
	end
	
end



function fix_sceneraio_structure(sname)
  for k, entity in pairs (game.surfaces[1].find_entities_filtered{name = sname}) do
	entity.minable = false
	entity.destructible = false
--	entity.force = game.forces["enemy"]
	end
end
function setup_enemy_structures()
	fix_sceneraio_structure("stone-wall")
	fix_sceneraio_structure("gate")
	
	local surface = game.surfaces[1]
	
	local jailbox = surface.find_entities_filtered{name = "steel-chest"}[1]
	global.jb_Jail_position = jailbox.position
	local jailbox2 = surface.find_entities_filtered{name = "iron-chest"}[1]
	global.jb_Jail_distance = distance_square(jailbox.position,jailbox2.position)
	global.jb_Jail_exit = jailbox2.position
	RevelaMapa(jailbox.position)
	local event = {event='add_mapflag',tick=300,tab_event={icon={type="item",name="gate"},text="Prisão",position=jailbox.position}}
	table.insert(global.jb_NextEvents,event)

	jailbox.destroy()
	jailbox2.destroy()

  local jailkey = surface.find_entities_filtered{name = "burner-inserter"}[1]
  global.jb_Jail_key = jailkey.position
  jailkey.destroy()
	
  local loot_chest= {}
  for k, entity in pairs (surface.find_entities_filtered{name = "logistic-chest-active-provider"}) do
    local position = entity.position
      table.insert(loot_chest, position)
    entity.destroy()
  end	
  global.jb_Loot_Chest = loot_chest
  RevelaMapa(loot_chest[1])
  local event = {event='add_mapflag',tick=300,tab_event={icon={type="item",name="logistic-chest-active-provider"},text="Loot",position=loot_chest[1]}}
  table.insert(global.jb_NextEvents,event)

  
  local replicator = surface.find_entities_filtered{name = "logistic-chest-requester"}[1]
  replicator.minable = false
  replicator.destructible = false
  global.jb_Replicator = replicator
  
  local destinator = surface.find_entities_filtered{name = "logistic-chest-passive-provider"}[1]
  destinator.minable = false
  destinator.destructible = false
  global.jb_Destinator = destinator

  local seller = surface.find_entities_filtered{name = "logistic-chest-buffer"}[1]
  seller.minable = false
  seller.destructible = false
  global.jb_SellerChest = seller

  
  local market = surface.find_entities_filtered{name = "market"}[1]
  market.minable = false
  market.destructible = false
  global.jb_Market = market
  local event = {event='add_mapflag',tick=300,tab_event={icon={type="virtual",name="signal-yellow"},text="Market",position=market.position}}
  table.insert(global.jb_NextEvents,event)


  local start_chest = surface.find_entities_filtered{name = "logistic-chest-storage"}[1]
  start_chest.destructible = false
  global.jb_InitialChest = start_chest

  local construct = surface.find_entities_filtered{name = "accumulator"}[1]
  global.jb_Construct_Limit_Y = construct.position.y
  construct.destroy()
  
end


function randomize_ore()
  local surface = game.surfaces[1]
  for k, ore in pairs (surface.find_entities_filtered{type = "resource"}) do
    ore.amount = ore.amount + rand(-5,5)
  end
end

function randomize_trees()
  local surface = game.surfaces[1]
  for k, ent in pairs(surface.find_entities_filtered{type= "tree"}) do
	if rand(1,5)==1 then ent.destroy() end
	end
end

------------------------------------SPAWN & ATTACKS -----------------------------------------------------------------------------
function CallFrenzyAttack()
local Min = global.JB_GROUP_LIMIT * 2
local Max = 1000
local Dist = 3000
local aliens = Max * game.forces.enemy.evolution_factor
if aliens < Min then aliens=Min end
local waypoints = global.jb_Waypoints
local target = waypoints[rand(#waypoints)]

local sent = game.surfaces[1].set_multi_command({
			command = {type=defines.command.attack_area, destination=target, radius=50, distraction = defines.distraction.by_anything },
			unit_count = aliens,
			force = game.forces.enemy,
			unit_search_distance = Dist,
		})
end



function prepare_enemy_list()
--           {unit="small-biter", spawn_points={{evolution_factor=0.0, weight=0.3}, {0.7, 0.0}}}
local ef = game.forces.enemy.evolution_factor
local Units = global.jb_Units
local Worms = global.jb_Worms
local elist = {}
for _,U in pairs (Units) do
    local name= U['unit'] 
    local SP = U['spawn_points']
	local emin = SP[1]['evolution_factor']
	local emax = SP[#SP]['evolution_factor']
	if SP[#SP]['weight']>0 then emax=2 end
	if ef>=emin and ef<emax then
	   table.insert(elist, name) 
	  -- game.players[1].print(#elist .. ' = ' .. name)
	   end
	end
global.jb_round_enemies = elist


local elist = {}
for _,W in pairs (Worms) do
	local e = game.entity_prototypes[W].build_base_evolution_requirement
	if ef>=e then
	   table.insert(elist, W) 
	   end
	end
global.jb_round_worms = elist

end



function finish_round_now()
local surface = game.surfaces[1]
local fe = game.forces.enemy
local power = 0

  local alive =  surface.find_entities_filtered{type = "unit", force=fe}
  for a,alien in pairs(alive) do
      if alien.valid and alien.health>0 then
	    power = power + alien.health
		alien.destroy()
		end end
  
  local alive =  surface.find_entities_filtered{type = "unit-spawner", force=fe}
  for a,alien in pairs(alive) do
      if alien.valid and alien.health>0 then
	    power = power + alien.health
		alien.destroy()
		end end
  
  local alive =  surface.find_entities_filtered{type = "turret", force=fe}
  for a,alien in pairs(alive) do
      if alien.valid and alien.health>0 then
		if not is_in_jail(alien) then
	    power = power + alien.health
		alien.destroy()
		end end end

global.jb_Enemy_Power_Bonus = power
RoundEnd()
end

function check_aliens_alive()
local surface = game.surfaces[1]
local fe = game.forces.enemy
  local alive =  surface.find_entities_filtered{type = "unit", force=fe, limit=1}[1] 
  if alive == nil then
	alive =  surface.find_entities_filtered{type = "unit-spawner",force=fe,limit=1}[1] 	end
  if alive == nil then
	alive =  surface.find_entities_filtered{type = "turret",force=fe,limit=1}[1] 	end  
  if alive == nil then 
	RoundEnd()
	end
end


function check_jail_prisoners()
  for _, player in pairs(game.connected_players) do
 		if player.character and player.character.valid then
			if not is_in_jail(player.character) then -- esta livre
			    local ItemsList = global.HoldPlayerItems[player.name] or {}
				if #ItemsList>0 then
					RestorePlayerItems(player)
				    end
				end
			end
     end
end

function check_cats_alive()

local f = game.forces.player
local surface = game.surfaces[1]
local respawn = f.get_spawn_position(surface)
local cats =  surface.find_entities_filtered{type = "unit", force=f}

if #cats<1 then return end

local waypoints = global.jb_Waypoints
local hero = GetPlayerNear(global.jb_Jail_key,3)
--local gupodoGato = surface.create_unit_group{position = global.jb_Jail_position , force=game.forces.player}
local go_to = defines.command.go_to_location
local command = {type = go_to, destination = respawn}

for x=1,#cats do
  local cat = cats[x]
  if distance(cat.position,respawn)>250 then
   if is_in_jail(cat) then 
      if hero~=nil then 
		 local command = {type = go_to, destination = respawn}
		 cat.set_command(command)
		 end
      end
	
	else
	   if rand(1,6)==1 then
		 local command = {type = go_to, destination = waypoints[rand(#waypoints)]}
		 cat.set_command(command)  end
	end
  end  
end
	
function calculate_wave_power(x)
  local c = 1.65
  local p = math.min(#game.connected_players, 7)
  local s = math.ceil((1.15^p)*(x^c)*1000 * global.Wave_Power_Mult )
--  if x % global.jb_BossRound == 0 then
  --  s = math.ceil(s/3) end
  s = s + global.jb_Enemy_Power_Bonus
  return s
end

function is_boss_round()
return global.jb_Round % global.jb_BossRound == 0
end

function spawn_units()

  local p = math.min(#game.connected_players,1)
  local int  = math.ceil
  local surface = game.surfaces[1]
  local round = global.jb_Round
  local group_limit = global.JB_GROUP_LIMIT + round *2
  local spawns = global.jb_Spawns_Pos     -- spawns positions
  local spawners = global.jb_Spawners    -- spawns entity names
  local units = global.jb_round_enemies
  local worms = global.jb_round_worms
  local power = calculate_wave_power(round)
  local groups = {}
  local group_count = 1
  local new_group = true
  local spawn
  local unit_count
  local group

 
    -- If This is a boss round  -- add spawners
  if is_boss_round() then
     local QS = int(p + round * 1.5)
	 if QS > #spawns then QS=#spawns end
	 for x=1, QS do
    	local spawner  = spawners[rand(#spawners)]
		local position = spawns[rand(#spawns)]
		local position = surface.find_non_colliding_position(spawner, position, 32, 2)
		if position~=nil then
		    surface.create_entity{name = spawner, position = position, force=game.forces.enemy}
			end
		end

	 if round>=global.jb_BossRound*2 then
     local QS = int(p + round)
	 for x=1, QS do
    	local worm  = worms[rand(#worms)]
		local position = spawns[rand(#spawns)]
		local position = surface.find_non_colliding_position(worm, position, 32, 2)
		if position~=nil then
		    surface.create_entity{name = worm, position = position, force=game.forces.enemy}
			end
		end
		end
		
	end
  
  
  
  while power > 0 do
    if new_group == true then
      spawn = spawns[rand(#spawns)]
      group = surface.create_unit_group{position = spawn, force=game.forces.enemy}
      groups[group_count] = group; 
	  group_count = group_count + 1
      new_group = false
      unit_count = 0
	  
    end
    local biter = units[rand(#units)]
    local position = surface.find_non_colliding_position(biter, spawn, 32, 2)
      if position then
        if unit_count <= group_limit then
		  local Alien = surface.create_entity{name = biter, position = position , force=game.forces.enemy}
		  local health = Alien.prototype.max_health
          group.add_member(Alien)
          power = power-health
          unit_count = unit_count + 1
		  if power<1 then break end
        else
          new_group = true
        end
      else
        break
      end
  end
  set_command(groups)
end


function is_in_jail(prisoner)
if not prisoner.valid then return false
  else return  distance_square(prisoner.position,global.jb_Jail_position)<global.jb_Jail_distance
	end
end

function set_command(groups)
  --if not (global.silo and global.silo.valid) then return end
  local waypoints = global.jb_Waypoints
  --local players = {}

  for _, player in pairs(game.connected_players) do
 		if player.character and player.character.valid then
			if not is_in_jail(player.character) then -- esta livre
			    table.insert(waypoints,player.character.position)
				end
			end
     end

  local def = defines
  local compound = def.command.compound
  local structure = def.compound_command.return_last
  local go_to = def.command.go_to_location
  local attack = def.command.attack
  
  for k, group in pairs (groups) do
    local commando =
	  {
        {type = go_to, destination = waypoints[rand(#waypoints)]},
      }
	  
--	if #players>1 then   --errooo
--	   table.insert(commando,{type = attack, target = players[rand(#players)]})
--	   else
	   table.insert(commando,{type = go_to, destination = waypoints[rand(#waypoints)]})
--	   end
	
    group.set_command
    {
      type = compound,
      structure_type = structure,
      commands = commando
    }
  end
end

function command_straglers()
local go_to = defines.command.go_to_location

  local waypoints = global.jb_Waypoints
    for _, player in pairs(game.connected_players) do
 		if player.character and player.character.valid then
			if not is_in_jail(player.character) then -- esta livre
			    table.insert(waypoints,position)
				end
			end
     end
  local command = {type = go_to, destination = waypoints[rand(#waypoints)]}
  for k, unit in pairs (game.surfaces[1].find_entities_filtered({type = "unit",force=game.forces.enemy})) do
    if not unit.unit_group then
      unit.set_command(command)
    end
  end
end

---------------------------------------------------------------------------------------------------------------------------


function RevelaMapa(posicao)
game.forces["player"].chart(game.surfaces[1], {lefttop = {posicao.x -10, posicao.y -10}, rightbottom = {posicao.x +10, posicao.y +10}})
end


function spawn_cats()
local cat_names = global.jb_CatNames
if not game.entity_prototypes[cat_names[1]] then return end
local f = game.forces.player
local surface = game.surfaces[1]
local respawn = f.get_spawn_position(surface)

local npc =  surface.find_entities_filtered{type = "unit", force=f}

local qt = 5- #npc
for x=1,qt do
  local cat = cat_names[rand(1,#cat_names)]
  local position = surface.find_non_colliding_position(cat, respawn, 32, 2)
  surface.create_entity{name = cat, position = position, force=f}
  end
  
end


function startGame()
global.jb_TimeToNextRound = global.jb_RestTimeBoss
set_market_offers()
spawn_cats()
end

function RoundStart()
for k=1,#global.jb_ChestLootToDestroy do
    global.jb_ChestLootToDestroy[k].destroy() end

global.jb_TimeToNextRound = 0
global.jb_MarkedTick = game.tick
global.jb_Round = global.jb_Round  + 1
--game.players[1].print('..preparando lista inimigos')
prepare_enemy_list()  -- atualiza global.jb_round_enemies 
--game.players[1].print('..preparando mercado')
set_market_offers()
game.print({"round-start", global.jb_Round}, colors.green)
--game.players[1].print('..spawn')
spawn_units()

if is_boss_round() then
   game.print({"boss-round-start", global.jb_Round}, colors.red)
   end
   
sound_all("utility/new_objective")
spawn_cats()

end


function CreateLootOnChest(chest,round,extras)
local commom  = 20 + extras*3
local good    = 10 + extras
 
local r = math.ceil(round * #global.Sorted_Item_Prices / global.JB_MAX_ROUNDS)
local ma = math.min(r+40,#global.Sorted_Item_Prices)
local mi = math.max(r-20,1)

local mag = math.min(ma+20,#global.Sorted_Item_Prices)
local mig = math.min(mi+10,#global.Sorted_Item_Prices,mag-40)


for c=1,good do
    local item  = global.Sorted_Item_Prices[rand(mig,mag)].name    -- ERRO AQUI field IR -- erro no replicador e no mercado
	local stackm = global.Items_Stack_Sizes[item]
	--if stackm==nil then stackm = game.item_prototypes["plastic-bar"].stack_size end
	if stackm~=nil then	
	if stackm>1 then stackm = math.random(math.floor(stackm/4),stackm) end
	local stack = {name=item, count=stackm}
    local invdest = chest.get_inventory(defines.inventory.chest)
	if not invdest.can_insert(stack) then break end
	invdest.insert(stack)	
	end
	end
 
for c=1,commom do
    local item  = global.Sorted_Item_Prices[rand(mi,ma)].name
	local stackm = global.Items_Stack_Sizes[item]
	--if stackm==nil then stackm = game.item_prototypes["plastic-bar"].stack_size end
	if stackm~=nil then
	if stackm>1 then stackm = math.random(math.floor(stackm/2),stackm) end
	local stack = {name=item, count=stackm}
    local invdest = chest.get_inventory(defines.inventory.chest)
	if not invdest.can_insert(stack) then break end
	invdest.insert(stack)	
	end
	end
	
end


function CreateLootBoxes()
local round = global.jb_Round 
local p = math.min(#game.connected_players, 5)
local extras = #game.connected_players - p
for k=1,p do
    local chest = game.surfaces[1].create_entity{name = "logistic-chest-active-provider", position = global.jb_Loot_Chest[k], force=game.forces.player}
	chest.minable = false
	table.insert(global.jb_ChestLootToDestroy,chest)
	CreateLootOnChest(chest,round,extras)
	end
end

function RoundEnd()
local rest_time=global.jb_RestTime 
if is_boss_round() then 
   rest_time=global.jb_RestTimeBoss
   CreateLootBoxes()
   sound_all("utility/achievement_unlocked")
   else
   sound_all("utility/scenario_message")
   end

local round = global.jb_Round   
global.jb_TimeToNextRound = rest_time + game.tick

local ef = game.forces.enemy.evolution_factor+ global.EVOLUTION_PER_ROUND
if ef>1 then ef=1 end
game.forces.enemy.evolution_factor = ef

local extracash = 0
if game.tick < rest_time + global.jb_MarkedTick then
   extracash = math.floor(global.skipped_multiplier*(rest_time+global.jb_MarkedTick-game.tick)*(1.15^global.jb_Round))
   end   

local cash = global.MONEY_PER_ROUND*round   
local catcash = 0

game.print({"round-end", round, format_number(cash),format_number(extracash)}, colors.green)   

if game.entity_prototypes[global.jb_CatNames[1]] then 
   local cats = #game.surfaces[1].find_entities_filtered{type = "unit", force=f}
   catcash = math.floor (cats * 500 * round / 2)
   game.print({"round-end-cats", cats, format_number(catcash)}, colors.green)   
   end
   
global.jb_TeamMoney = global.jb_TeamMoney + cash + extracash + catcash
global.jb_MarkedTick= 0
	 
GiveTeamXP(round*global.XP_PER_ROUND)
end

------------------------------------------------------------------------------------------------------------------

script.on_init(function()
init_globals()
setup_executioners()
setup_enemy_structures()
init_unit_settings()
setup_techs()
randomize_ore()
randomize_trees()
scenario_starting_equipment(global.jb_InitialChest)
startGame()
end)



script.on_event(defines.events.on_player_created, function(event)
  local player = game.players[event.player_index]
  give_spawn_equipment(player)
  give_starting_equipment(player)
  if player.index == 1 then
    game.show_message_dialog({text = {"welcome-to-jail-break"}})
  else
    player.print({"welcome-to-jail-break"})
  end
end)


script.on_event(defines.events.on_player_joined_game, function(event)
  local player = game.players[event.player_index]
  gui_init(player)
end)








---------------------------------------Player GUI--------------------------------------------------------
function close_all_gui()
for p,player in pairs (game.players) do
	if player.valid then
		if player.gui.center.team_upgrade_frame then
			player.gui.center.team_upgrade_frame.destroy() end
		if player.gui.center.jb_price_table then
			player.gui.center.jb_price_table.destroy() end	
		end
	end
end


function create_tech_gui(gui)
  local player = game.players[gui.player_index]
  if gui.team_upgrade_frame then
    gui.team_upgrade_frame.destroy()
    return
  end
  local team_upgrades = gui.add{type = "frame", name = "team_upgrade_frame", caption = {"buy-upgrades"}, direction = "vertical"}
  team_upgrades.style.visible = true
  team_upgrades.style.title_bottom_padding = 2
  local money_table = team_upgrades.add{type = "table", name = "money_table", column_count = 2}
  money_table.style.column_alignments[2] = "right"
  local label = money_table.add{type = "label", name = "force_money_label", caption = {"force-money"}}
  label.style.font = "default-semibold"
  local cash = money_table.add{type = "label", name = "force_money_count", caption = format_number(global.jb_TeamMoney)}.style
  cash.font = "default-bold"
  cash.font_color = colors.yellow
  local scroll = team_upgrades.add{type = "scroll-pane", name = "team_upgrade_scroll"}
  scroll.style.maximal_height = 450
  local upgrade_table = scroll.add{type = "table", name = "upgrade_table", column_count = 2}
  upgrade_table.style.horizontal_spacing = 0
  upgrade_table.style.vertical_spacing = 0
  update_upgrade_listing(upgrade_table)
end



function update_upgrade_listing(gui)
local techs  = global.jb_Tech_List  
local utechs = global.jb_Unique_Tech_List  

  for k,tech in pairs (techs) do
    local level = get_current_tech_level(tech)
	local tech_lv = tech .. '-'..level
	local price = global.jb_Tech_Prices[tech_lv]
	
	if price ~= nil and price >0 then
    local txt = game.technology_prototypes[tech_lv].localised_name

    if not gui[tech_lv] then
      local sprite = gui.add{type = "sprite-button", name = tech_lv, sprite = "technology/"..tech_lv, tooltip = {"purchase"}, style = "play_tutorial_button"}
      sprite.style.minimal_height = 75
      sprite.style.minimal_width = 75
      local flow = gui.add{type = "frame", name = tech.."_flow", direction = "vertical"}
      flow.style.maximal_height = 75
      local another_table = flow.add{type = "table", name = tech.."_label_table", column_count = 1}
      another_table.style.vertical_spacing = 2
      local label = another_table.add{type = "label", name = tech.."_name", caption = txt}
      label.style.font = "default-bold"
	  label.style.font_color = colors.pink
      local pricelb = another_table.add{type = "label", name = tech.."_price", caption = {"upgrade-price", format_number(price)}}
	  pricelb.style.font_color = colors.yellow
      local level = another_table.add{type = "label", name = tech.."_level", caption = {"upgrade-level", level-1}}
    else
      gui[tech.."_flow"][tech.."_label_table"][tech.."_price"].caption = {"upgrade-price", format_number(price)}
      gui[tech.."_flow"][tech.."_label_table"][tech.."_level"].caption = {"upgrade-level", level-1}
    end
	end
  end
  
  
  for k,tech in pairs (utechs) do
    local valido = tech_has_prereq(tech)
	if valido then
	local txt = game.technology_prototypes[tech].localised_name
    if not gui[tech] then
      local sprite = gui.add{type = "sprite-button", name = tech, sprite = "technology/"..tech, tooltip = {"purchase"}, style = "play_tutorial_button"}
      sprite.style.minimal_height = 75
      sprite.style.minimal_width = 75
      local flow = gui.add{type = "frame", name = tech.."_flow", direction = "vertical"}
      flow.style.maximal_height = 75
      local another_table = flow.add{type = "table", name = tech.."_label_table", column_count = 1}
      another_table.style.vertical_spacing = 2
      local label = another_table.add{type = "label", name = tech.."_name", caption = txt}
      label.style.font = "default-bold"
	  label.style.font_color = colors.green
      local pricelb = another_table.add{type = "label", name = tech.."_price", caption = {"upgrade-price", format_number(global.jb_Tech_Prices[tech])}}
	  pricelb.style.font_color = colors.yellow
    else
      gui[tech.."_flow"][tech.."_label_table"][tech.."_price"].caption = {"upgrade-price", format_number(global.jb_Tech_Prices[tech])}
    end
	end
  end  
  
  
end

function get_current_tech_level(tech)
local f=game.forces.player
local l
for lv=1, 15 do  
   name= tech..'-'..lv
   if (not game.technology_prototypes[name]) or 
      (not f.technologies[name].researched) then
	  l=lv
	  break 
	  end
   end
return l
end


function tech_has_prereq(tech)
local f=game.forces.player
local valido = not f.technologies[tech].researched
local pre = game.technology_prototypes[tech].prerequisites

if valido then	
for k,req in pairs (pre) do
    if not f.technologies[req.name].researched then 
		valido=false
		break
		end
	end end
return valido
end

function create_price_table_info(player)

	if player.gui.center.jb_price_table then
	   player.gui.center.jb_price_table.destroy() 
	   return
	   end
	   
	local frame = player.gui.center.add {type = "frame", name = "jb_price_table", caption={"jb_price_table"}, direction = "vertical"}
	local price_scroll = frame.add{type = "scroll-pane", name= "price_table_scroll", vertical_scroll_policy="auto", horizontal_scroll_policy="auto"}
	price_scroll.style.minimal_height = 400
	price_scroll.style.maximal_height = 650
	price_scroll.style.minimal_width  = 530
	price_scroll.style.maximal_width  = 530

	local tab_scroll = price_scroll.add{type = "table", column_count = 12}
	
	for i=1, #global.Sorted_Item_Prices do
		local name = global.Sorted_Item_Prices[i].name
		local price = global.Sorted_Item_Prices[i].price
        local txt = game.item_prototypes[name].localised_name
		tab_scroll.add{type = "sprite-button",name = "jb_bt"..name, sprite ="item/"..name,tooltip =txt ,style = mod_gui.button_style}
		tab_scroll.add{type = "label", caption =format_number(price)}
		end
end

local function create_map_info(player)	
	if player.gui.left.jail_break_map_info then
	   player.gui.left.jail_break_map_info.destroy() 
	   return
	   end
	local frame = player.gui.left.add {type = "frame", name = "jail_break_map_info", direction = "vertical"}
	local t = frame.add {type = "table", column_count = 1}	
	
	local tt = t.add {type = "table", column_count = 3}
	local l = tt.add {type = "label", caption = " --Jail Break-- "}
	l.style.font = "default-frame"
	l.style.font_color = {r=0.6, g=0.3, b=0.99}
	l.style.top_padding = 6	
	l.style.bottom_padding = 6
	
	local l = tt.add {type = "label", caption = " *by MFerrari* "}
	l.style.font = "default"
	l.style.font_color = {r=0.99, g=0.99, b=0.2}
	l.style.minimal_width = 280	
	
	local b = tt.add {type = "button", caption = "X", name = "close_map_info", align = "right"}	
	b.style.font = "default"
	b.style.minimal_height = 30
	b.style.minimal_width = 30
	b.style.top_padding = 2
	b.style.left_padding = 4
	b.style.right_padding = 4
	b.style.bottom_padding = 2
	
	local tt = t.add {type = "table", column_count = 1}
	local frame = t.add {type = "frame"}
	local l = frame.add {type = "label", caption = global.jb_map_info}
	l.style.single_line = false	
	l.style.font_color = {r=0.95, g=0.95, b=0.95}	
end

function time_to_that_tick(ThatTick)
  if game.tick > ThatTick then return '---' end
  return format_time(ThatTick - game.tick)
end
function time_from_that_tick(ThatTick)
  if game.tick < ThatTick then return '---' end
  return format_time(game.tick-ThatTick)
end


function gui_init(player)
  local gui = mod_gui.get_frame_flow(player)
  if gui.jb_round_frame then
    gui.jb_round_frame.destroy()
  end
  update_wave_frame(gui)
  
  create_map_info(player)	

  local button_flow = mod_gui.get_button_flow(player)
  local button = button_flow.jail_break_main_button
  if not button then
    button_flow.add
    {
      type = "sprite-button",
      name = "jail_break_main_button",
      style = mod_gui.button_style,
      sprite = "entity/big-worm-turret",
      tooltip = {"jail_break_main_button_tooltip"}
    }
  end
  
  
  local button_flow = mod_gui.get_button_flow(player)
  local button = button_flow.jail_break_map_info_bt
  if not button then
    button_flow.add
    {
      type = "button",
      name = "jail_break_map_info_bt",
      style = mod_gui.button_style,
      --sprite = "entity/big-worm-turret",
	  caption = '?',
      tooltip = {"jb-about-button-tooltip"}
    }
  end
  
end

function update_wave_frame(gui)
  if not gui.valid then return end
  
  local frame = gui.jb_round_frame
  if not frame then 
     frame = gui.add{type = "frame", name = "jb_round_frame", caption = {"wave-frame"}, direction = "vertical"}

     local info_table = frame.add{type = "table", name = "jb_inf_table", column_count = 2}
 
	 local i1 = info_table.add{type = "label", name = "jb_evolution_factor"}
	 local i2 = info_table.add{type = "label", name = "jb_time_played"}
	 local i3 = info_table.add{type = "label", name = "jb_kills_u"}
	 local i4 = info_table.add{type = "label", name = "jb_kills_s"}
	 local i5 = info_table.add{type = "label", name = "jb_kills_w"}
	 local i6 = info_table.add{type = "label", name = "jb_current_round"}
		 

     local time_table = frame.add{type = "table", name = "jb_time_table", column_count = 2}
     local t1 = time_table.add{type = "label", name = "jb_timer"}
	 local b1 = time_table.add{type = "button",name = "jb_send_next_wave", caption = {"send-next-wave"}, tooltip = {"send-next-wave-tooltip"},style = mod_gui.button_style}

	 local money_table = frame.add{type = "table", name = "jb_money_table", column_count = 6}
		money_table.add{type = "label", name = "jb_force_money_label", caption = {"force-money"}}
		local cash = money_table.add{type = "label", name = "force_money_count"}
		money_table.add{type = "sprite-button",name = "jb_cash_out_button", sprite = "item/coin", tooltip = {"cash_out_button_tooltip"},style = mod_gui.button_style}
		money_table.add{type = "sprite-button",name = "jb_deposit_button", sprite = "virtual-signal/signal-anything", tooltip = {"deposit_button_tooltip"},style = mod_gui.button_style}
		money_table.add{type = "sprite-button",name = "jb_shop_button", sprite = "entity/market", tooltip = {"open_shop_button_tooltip"},style = mod_gui.button_style}
		local b0 = money_table.add{type = "sprite-button",sprite = "item/science-pack-1",name = "jb_buy_techs",  tooltip = {"jb_buy_techs_tooltip"},style = mod_gui.button_style}
		--local b0 = money_table.add{type = "button",name = "jb_about_bt", caption = '?', tooltip = {"jb-about-button-tooltip"},style = mod_gui.button_style}
	 frame.style.visible = true
	 
	 i1.style.font_color=colors.lightred
	 i2.style.font_color=colors.yellow
	 i6.style.font_color=colors.lightblue
	 cash.style.font = "default-semibold"
	 cash.style.font_color = colors.yellow
	 end
  
  -- update values
     local info_table = frame.jb_inf_table
	 local i1 = info_table.jb_evolution_factor
	 local i2 = info_table.jb_time_played
	 local i3 = info_table.jb_kills_u
	 local i4 = info_table.jb_kills_s
	 local i5 = info_table.jb_kills_w
	 local i6 = info_table.jb_current_round
     local time_table = frame.jb_time_table
     local t1 = time_table.jb_timer
	 local b1 = time_table.jb_send_next_wave
	 local money_table = frame.jb_money_table
     local cash = money_table.force_money_count
  
	 i1.caption =  FormatEvolution()
	 i2.caption =  {"time_played",time_from_that_tick(0)}
	 i3.caption =  {"kills_unit",global.jb_UnitKills}
	 i4.caption =  {"kills_spawner",global.jb_SpawnerKills}
	 i5.caption =  {"kills_worm",global.jb_WormKills}
	 i6.caption =  {"current-round", global.jb_Round}
	 cash.caption = format_number(global.jb_TeamMoney)
	 
  if global.jb_TimeToNextRound >0 then
      t1.caption =  {"time_to_next_round", time_to_that_tick(global.jb_TimeToNextRound)}
	  b1.style.visible =true
	  else
      t1.caption =  {"elapsed_time_from", time_from_that_tick(global.jb_MarkedTick)}
	  b1.style.visible =false
	 end
 end

 
 
function update_connected_players()
  for k, player in pairs (game.connected_players) do
     local gui = mod_gui.get_frame_flow(player)
	 if gui.valid then
	 if gui.jb_round_frame.style.visible then 
	    update_wave_frame(gui)
		end end
		
	 local frame = player.gui.center.team_upgrade_frame
     if frame and frame.valid then
		update_upgrade_listing(frame["team_upgrade_scroll"]["upgrade_table"])
        end	
  end
end
 
 

function Player_Cash_Out(player)
 if global.jb_TeamMoney < global.MIN_TO_CASH_OUT then
	player.print({"cant-cash-out"},colors.red)
	return end

local cash = 100	
if global.jb_TeamMoney>10000 then cash = 1000 
   elseif global.jb_TeamMoney>100000 then cash = 10000 end 

local inventory = player.get_inventory(defines.inventory["player_main"])
if inventory.can_insert{name="coin", count=cash} then
	player.character.insert{name="coin", count=cash}
	global.jb_TeamMoney = global.jb_TeamMoney - cash
	game.print({"player-cash-out",player.name, format_number(cash)}, colors.yellow)
	update_connected_players()
	end
end

function Player_Deposit(player)
 local coins = player.get_item_count("coin")
 if coins >= 1  then
    local inventories = {"quickbar","main"}
    for i=1,#inventories do
       local inventory = player.get_inventory(defines.inventory["player_"..inventories[i]])
       inventory.remove({name="coin", count=coins})
    end
 global.jb_TeamMoney = global.jb_TeamMoney + coins
 player.print({"player-deposit",coins},colors.yellow)
 update_connected_players()
 end
end
 
   -- CLICK
 
script.on_event(defines.events.on_gui_click, function(event)
  local gui = event.element
  local player = game.players[event.player_index]

  if gui.name == "jb_send_next_wave" then
     local skipped = math.floor(global.skipped_multiplier*(global.jb_TimeToNextRound - game.tick)*(1.15^global.jb_Round))
     global.jb_TeamMoney = global.jb_TeamMoney + skipped
     game.print({"sent-next-wave", player.name,format_number(skipped)}, colors.yellow)
	RoundStart() 
    update_connected_players()
    return
  end
  

  if gui.name == "jail_break_main_button" then
    local gui =  mod_gui.get_frame_flow(player)
    gui.jb_round_frame.style.visible = not gui.jb_round_frame.style.visible
    return
  end
  
  if gui.name == "jb_cash_out_button" then
	 Player_Cash_Out(player)
	 return
	 end
     
  if gui.name == "jb_deposit_button" then
     Player_Deposit(player)
	 return
	 end
  
  if gui.name == "jb_buy_techs" then
     create_tech_gui(player.gui.center)
	 return
	 end	

  if gui.name == "jail_break_map_info_bt" then
     create_map_info(player) 
	 return
	 end	
	 
	 
  if gui.name == "close_map_info" then
     if player.gui.left.jail_break_map_info then 
	    player.gui.left.jail_break_map_info.destroy() end	
		return
		end

  if gui.name == "jb_shop_button" then
     --set_market_offers()
     create_price_table_info(player) 
	 return	 
  end
	 
	 
 if global.jb_Tech_Prices[gui.name]~=nil then
     buy_tech(player,gui.name) 
	 return	 
	end

 
end)
-------------------------------------------------------------------------------------------------------------------



function buy_tech(player,tech) 
local price = global.jb_Tech_Prices[tech]
local name  = game.technology_prototypes[tech].localised_name
   if global.jb_TeamMoney >= price then
      global.jb_TeamMoney = global.jb_TeamMoney -price
	  game.forces.player.technologies[tech].researched = true
	  game.print({"purchased-tech", player.name, name,format_number(price)},colors.green)
	  create_tech_gui(player.gui.center) -- closes
	  update_connected_players()
      sound_all("utility/research_completed")
	  else
	  player.print({"cant-cash-out"},colors.red)
	  end
end


-------------------Player Inventory------------------------------------------------------------------------------------------------

function RemovePlayerItems(player)
  local inventories = {"quickbar","main","guns","ammo","armor","tools","trash"}
  local ItemsList = global.HoldPlayerItems[player.name] or {}
  for i=1,#inventories do
    local inventory = player.get_inventory(defines.inventory["player_"..inventories[i]])
	    for k = 1, #inventory do
			local stack = inventory[k]
			if stack and stack.valid and stack.valid_for_read then
				local copy_of_item_stack = { name = stack.name, count = stack.count, health = stack.health or nil, durability = stack.durability or nil }
				table.insert(ItemsList,copy_of_item_stack)
				inventory.remove(stack)
				end
            end
    end

global.HoldPlayerItems[player.name] = ItemsList
end

function CleanPlayerItems(player)
global.HoldPlayerItems[player.name] = {}
end

function RestorePlayerItems(player)
if not global.HoldPlayerItems[player.name] then return end
if not player.character or not player.character.valid then return end

for i, items in pairs (global.HoldPlayerItems[player.name]) do
	player.insert(items)
	end
if #global.HoldPlayerItems[player.name]>1 then
   player.surface.create_entity{name = "flying-text", position = player.character.position, text = "+ITEMS", color = colors.pink}
   end
CleanPlayerItems(player)	
end


---------------------------------------------------------------------------------------------------------------------------

function FeedTheHive(name,position)
   local ef = game.forces.enemy.evolution_factor+global.EVOLUTION_PER_KILL
   if ef>1 then ef=1   end
   game.forces.enemy.evolution_factor = ef
   game.surfaces[1].create_entity{name = "flying-text", position = position, text = "+ALIEN FOOD", color = colors.pink}
   game.print({"someone-fed-the-hive",name}, colors.pink);
   
   local GP_P = math.floor(global.jb_TeamMoney * global.GP_DEATH_PENALTY/100)
   game.print({"gp-lost",GP_P} , colors.yellow)
   global.jb_TeamMoney = global.jb_TeamMoney - GP_P
end



script.on_event(defines.events.on_pre_player_died, function(event)
local player = game.players[event.player_index]
local name = player.name
local position = player.character.position

RemovePlayerItems(player)

if not is_in_jail(player.character) then
   -- was free - goes to jail
   global.jb_WhereYouRespawn[name] = global.jb_Jail_position
   
   -- kill nearby units
   for key, entity in pairs(player.surface.find_entities_filtered({type='unit', area ={{position.x-5,position.y-5},{position.x+5,position.y+5}}, force="enemy"})) do
       if entity.valid then
	   entity.die() end end
   
   else --  was executed in jail
   global.jb_WhereYouRespawn[name] = nil
   CleanPlayerItems(player)
  
   -- aliens feed
   FeedTheHive(name,position)
   end
end)



script.on_event(defines.events.on_player_respawned, function(event)
  local player = game.players[event.player_index]
  local name = player.name
   
  if global.jb_WhereYouRespawn[name] ~=nil then 
     local position = player.surface.find_non_colliding_position("steel-chest", global.jb_WhereYouRespawn[name], 10, 2)
     player.teleport(position, player.surface)
	 game.print({"player-captured",name}, colors.orange)
		if global.jb_TickToKill==0 then global.jb_TickToKill = game.tick + global.jb_TimeToExecute end
	 else
	 give_spawn_equipment(player)
	 end
end)




script.on_event(defines.events.on_entity_died, function(event)
 
if event.force == nil then return end
local force=event.force.name  -- force that kill

local died = event.entity
local surface = died.surface

if died.force.name == 'enemy' and force~='neutral' and force~='enemy' then --aliens
	local GP = died.prototype.max_health
		if died.type == 'unit' then
			global.jb_UnitKills    = global.jb_UnitKills +1
		elseif died.type == 'unit-spawner' then
			global.jb_SpawnerKills = global.jb_SpawnerKills +1
			GP = GP * 3
		elseif died.type == 'turret' then
			global.jb_WormKills    = global.jb_WormKills +1
			GP = GP * 2
		end	

	GP = math.ceil(global.GP_Mult * GP/100) 
	if GP > 999999 then GP=999999 end
	if GP<1 then GP=1 end

	global.jb_TeamMoney    = global.jb_TeamMoney + GP  
	surface.create_entity{name = "flying-text", position = died.position, text = "$"..GP, color = colors.yellow}
	end
  
  
if is_in_table (global.jb_CatNames, died.name) then
   if is_in_jail(died) then	
		FeedTheHive(game.entity_prototypes[died.name].localised_name,died.position)
		else
		spawn_cat_in_jail(died.name)
		end
	end
  
--  if entity_type == "rocket-silo" then
 --   rocket_died(event)
  --  return
 -- end
end)



function spawn_cat_in_jail(name)
game.print({'cat-captured'}, colors.red)
if global.jb_TickToKill==0 then global.jb_TickToKill = game.tick + global.jb_TimeToExecute end
local f = game.forces.player
local surface = game.surfaces[1]
local position = surface.find_non_colliding_position(name, global.jb_Jail_position, 32, 2)
surface.create_entity{name = name, position = position, force=f}
end



function On_Built(event)

local ent = event.created_entity
local force = ent.force
	
	
local X = ent.position.x
local Y = ent.position.y
	
	if Y < global.jb_Construct_Limit_Y then  -- 
		local surface = ent.surface
		surface.spill_item_stack( {x = X, y= Y}, {name=ent.name, count=1}, false)
		ent.destroy()
		surface.create_entity{name = "flying-text", position = {x = X, y= Y}, text = {"cant-build-here"}, color = colors.red}
		end
end
local build_events = {defines.events.on_built_entity, defines.events.on_robot_built_entity}
script.on_event(build_events, On_Built)	



script.on_event(defines.events.on_research_finished, function(event)
local tech = event.research.name
close_all_gui()
end)